//
//  BBDXFBlock.h
//  DXFReader
//
//  Created by Allan Daly on 3/25/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFBlock.h
 @abstract		Defines the BBDXFBlock to represent block definition data from an ASCII DXF file.
 @discussion	This header defines class BBDXFBlock.
 
 BBDXFBlock represents the block definition information contained in an ASCII DXF file. 
 */

#import <Cocoa/Cocoa.h>
#import "BBDXFEntityContainer.h"

@class BB3DPoint;
@class BBDXFEntity;

// BBDXFBlock object HeaderDoc info
/*!
 @abstract		Defines the BBDXFBlock object to hold DXF BLOCK definition data.
 
 @discussion	BBDXFBlock represents the definition of a BLCOK from information
 contained in an ASCII DXF file. 
 */
@interface BBDXFBlock : BBDXFEntityContainer {
	/*!
	 @abstract		String value indicating the entity handle.
	 @discussion	Contains the data from DXF entity group code 5. This is a text string of up to 16 hexadecimal digits.
	 This value is fixed for the life of the entity and will never change. It is a unique identifier that
	 can be used to refer to a single entity.
	 
	 Group code 5: handle, string, has no default value
	 */
	NSString *  handle;			// group code 5:  text string of up to 16 hexadecimal digits (fixed - will never change)
	
	/*!
	 @abstract		String value indicating the owner's entity handle.
	 @discussion	Contains the data from DXF entity group code 330. This is a text string of up to 16 hexadecimal digits.
	 This points to the owner object for this block.
	 
	 Group code 330: ownerHandle, string, has no default value
	 */
	NSString *  ownerHandle;			// group code 330:  text string of up to 16 hexadecimal digits (fixed - will never change)
	
	/*!
	 @abstract		String value indicating the layer name for the entity.
	 @discussion	Contains the data from DXF entity group code 8. This is a text string indicating the layer name for the entity.
	 
	 Group code 8:	layer name, string, no default value
	 */
	NSString *  layerName;		// group code 8:  text string indicating the layer name

	/*!
	 @abstract		String value indicating the Xref path name.
	 @discussion	Contains the data from DXF entity group code 1. This is a text string containing
	 the Xref path name.
	 
	 Group code 1: Xref path name, string, has no default value
	 */
	NSString *  xrefPath;
	
	/*!
	 @abstract		String value indicating the block name.
	 @discussion	Contains the data from DXF entity group code 2. This is a text string containing
	 the block name.
	 
	 Group code 2: block name, string, has no default value
	 */
	NSString *  name;			// group code 2:  block name, text string

	/*!
	 @abstract		String value indicating the other block name.
	 @discussion	Contains the data from DXF entity group code 3. This is a text string containing
	 the other block name.
	 
	 Group code 3: other block name, string, has no default value
	 */
	NSString *  otherName;
	
	/*!
	 @abstract		String value indicating the block description (optional).
	 @discussion	Contains the data from DXF entity group code 4. This is a text string containing
	 description text for the block.
	 
	 Group code 4: block description, no default value
	 */
	NSString *  description;
	
	/*!
	 @abstract		Integer that provides "flag" information about the block.
	 @discussion	Contains the data from DXF entity group code 70. This is an integer value indicating the block properties.
	 
	 Group code 70:  block-type flags (bit coded, may be combined)
	 <li>a 16-bit integer value</li>
	 <li>indicating the block "flag" value</li>
	 <li>0 = indicates that none of the following flags apply</li>
	 <li>1 = This is an anonymous block generated by hatching, associative dimensioning, other internal operations, or an application</li>
	 <li>2 = This block has non-constant attribute definitions (this bit is not set if the block has any attribute definitions that are constant, or has no attribute definitions at all)</li>
	 <li>4 = This block is an external reference (xref)</li>
	 <li>8 = This block is an xref overlay</li>
	 <li>16 = This block is externally dependent</li>
	 <li>32 = This is a resolved external reference, or dependent of an external reference (ignored on input)</li>
	 <li>64 = This definition is a referenced external reference (ignored on input)</li>
	 */
	int blockFlag;

	/*!
	 @abstract		3D point indicating the block base point.
	 @discussion	Contains the data from DXF entity group codes 10, 20, 30. This is a 3D point indicating the block base point. 
	 
	 Group codes 10, 20, 30:  base point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * basePoint;		// group codes 10, 20, 30 indicating x, y, z values for base point

}

/*!
 @abstract	<i>handle</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * handle;

/*!
 @abstract	<i>ownerHandle</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * ownerHandle;

/*!
 @abstract	<i>layerName</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * layerName;

/*!
 @abstract	<i>xrefPath</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * xrefPath;

/*!
 @abstract	<i>name</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * name;

/*!
 @abstract	<i>otherName</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * otherName;

/*!
 @abstract	<i>description</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * description;

/*!
 @abstract	<i>basePoint</i> instance variable accessible through standard methods.
 */
@property (retain) BB3DPoint * basePoint;

/*!
 @abstract	<i>blockFlag</i> instance variable accessible through standard methods.
 */
@property int blockFlag;

@end
