//
//  BBDXFEntity.h
//  DXFReader
//
//  Created by Allan Daly (allan@brown-bird.net) on 1/30/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFEntity.h
 
 @abstract		Defines abstract class BBDXFEntity, which holds the common entity data elements.
 
 @discussion	BBDXFEntity is an abstract class that provides a structure for the data elements
				common to all AutoCAD DXF entities. This class serves as the parent class for
				all of the specific entity types defined in subclasses.
 */
#import <Cocoa/Cocoa.h>
#import "BB3DPoint.h"

/*!
 @abstract		Defines abstract class BBDXFEntity to hold the common DXF entity data.
 
 @discussion	BBDXFEntity is an abstract class that provides a structure for the data elements
				common to all AutoCAD DXF entities. This class serves as the parent class for
				all of the specific entity types defined in subclasses.

				Instance variables defined in BBDXFEntity are based on the AutoCAD 2009 DXF Reference
				available at <a href="http://images.autodesk.com/adsk/files/acad_dxf.pdf">http://images.autodesk.com/adsk/files/acad_dxf.pdf</a>
				as of 1/31/2009. The specific names used here were chosen to be consistent with this reference
				and use the AutoCAD terminology wherever possible.
 
				This implementation is not a complete set of the common DXF entity data items.
				See the AutoCAD 2009 DXF Reference listed above for the complete set.
 */
@interface BBDXFEntity : NSObject {

	/*!
	 @abstract		String value indicating the entity type.
	 @discussion	Contains the data from DXF entity group code 0. This is a text string indicating the entity type.
	 
					Group code 0:  entity type, string, has no default value
	 */
	NSString *  entityType;		// group code 0:  text string indicating the entity type

	/*!
	 @abstract		String value indicating the entity handle.
	 @discussion	Contains the data from DXF entity group code 5. This is a text string of up to 16 hexadecimal digits.
					This value is fixed for the life of the entity and will never change. It is a unique identifier that
					can be used to refer to a single entity.
	 
					Group code 5: handle, string, has no default value
	 */
	NSString *  handle;			// group code 5:  text string of up to 16 hexadecimal digits (fixed - will never change)

	/*!
	 @abstract		String value indicating the entity line type name.
	 @discussion	Contains the data from DXF entity group code 6. This is a text string indicating the entity line type name.
	 
					Group code 6: layer type name, string
								  <li>defaults to BYLAYER</li>
								  <li>special type BYBLOCK</li>
	 */
	NSString *	lineTypeName;	// group code 6:  text string indicating the line type name
								//				  defaults to BYLAYER
								//				  specialy type BYBLOCK valid

	/*!
	 @abstract		String value indicating the layer name for the entity.
	 @discussion	Contains the data from DXF entity group code 8. This is a text string indicating the layer name for the entity.
	 
					Group code 8:	layer name, string, no default value
	 */
	NSString *  layerName;		// group code 8:  text string indicating the layer name
	
	/*!
	 @abstract		16-bit (regular int) value indicating the color number.
	 @discussion	Contains the data from DXF entity group code 62. This is an integer value indicating the color number.
	 
					Group code 62:	color number, integer
									<li>(-255 to 256 valid)</li>
									<li>zero indicates BYBLOCK, 256 indicates BYLAYER</li>
									<li>a negative value indicates that the layer is turned off</li>
									<li>defaults to BYLAYER (value 256)</li>
	 */
	int			colorNumber;	// group code 62: 16-bit integer value indicating the color number (-255 to 256 valid)
								//                zero indicates BYBLOCK, 256 indicates BYLAYER
								//				  a negative value indicates that the layer is turned off
								//				  defaults to BYLAYER (value 256)	
}

/*!
 @abstract	<i>entityType</i> instance variable accessible through standard methods.
*/
@property (copy) NSString * entityType;

/*!
 @abstract	<i>handle</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * handle;

/*!
 @abstract	<i>lineTypeName</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * lineTypeName;

/*!
 @abstract	<i>layerName</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * layerName;

/*!
 @abstract	<i>colorNumber</i> instance variable accessible through standard methods.
 */
@property int colorNumber;

@end
