//
//  BBDXFEntityInsert.h
//  DXFReader
//
//  Created by Allan Daly (allan@brown-bird.net) on 4/3/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFEntityInsert.h
 
 @abstract		Subclass of BBDXFEntity. Defines class BBDXFEntityInsert, which holds the INSERT entity data elements.
 
 @discussion	BBDXFEntityInsert is a subclass of BBDXFEntity that provides a structure for the data elements
 defined in INSERT AutoCAD DXF entities. The data elements in this class plus those in the parent
 class make up all of the data found in the INSERT entity definition. (At this time, this implementation
 is not complete. See general discussion for more details.)
 */
#import <Cocoa/Cocoa.h>
#import "BBDXFEntity.h"

@class BB3DPoint;

/*!
 @abstract		Defines abstract class BBDXFEntityInsert to hold the INSERT DXF entity data.
 
 @discussion	BBDXFEntityInsert is a subclass of BBDXFEntity that provides a structure for the data elements
 in INSERT AutoCAD DXF entities.
 
 Instance variables defined in BBDXFEntityLine are based on the AutoCAD 2009 DXF Reference
 available at <a href="http://images.autodesk.com/adsk/files/acad_dxf.pdf">http://images.autodesk.com/adsk/files/acad_dxf.pdf</a>
 as of 1/31/2009. The specific names used here were chosen to be consistent with this reference
 and use the AutoCAD terminology wherever possible.
 
 This implementation is not a complete set of the common DXF entity data items.
 See the AutoCAD 2009 DXF Reference listed above for the complete set.
 */
@interface BBDXFEntityInsert : BBDXFEntity {

	/*!
	 @abstract		String value indicating the block name that the INSERT refers to.
	 @discussion	Contains the data from DXF entity group code 2. This is a text string containing
	 the block name referred to by this INSERT.
	 
	 Group code 2: block name (reference), string, has no default value
	 */
	NSString *  name;			// group code 2:  block name, text string
	
	/*!
	 @abstract		3D point indicating the INSERT insertion point.
	 @discussion	Contains the data from DXF entity group codes 10, 20, 30. This is a 3D point indicating the insertion point. 
	 
	 Group codes 10, 20, 30:  insertion point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * insertionPoint;
	
	/*!
	 @abstract		Double value x scale factor.
	 @discussion	Contains the data from DXF entity group code 41. This is a double value indicating the x scale factor.
	 
	 Group code 41:  x scale factor, double, default = 1
	 */
	double xScaleFactor;
	
	/*!
	 @abstract		Double value y scale factor.
	 @discussion	Contains the data from DXF entity group code 42. This is a double value indicating the y scale factor.
	 
	 Group code 42:  y scale factor, double, default = 1
	 */
	double yScaleFactor;
	
	/*!
	 @abstract		Double value z scale factor.
	 @discussion	Contains the data from DXF entity group code 43. This is a double value indicating the z scale factor.
	 
	 Group code 43:  z scale factor, double, default = 1
	 */
	double zScaleFactor;

	/*!
	 @abstract		Double value indicating block rotation (in degrees).
	 @discussion	Contains the data from DXF entity group code 50. This is a double value indicating the block rotation (in degrees).
	 
	 Group code 50:  block rotation (in degrees), double, default = 0
	 */
	double rotation;
	
	/*!
	 @abstract		Integer that indicates the column count for the insert (array?).
	 @discussion	Contains the data from DXF entity group code 70. This is an integer value indicating the
	 number of columns in the insert (array?).
	 
	 Group code 70:  column count, integer, default = 1
	 */
	int columnCount;	
	
	/*!
	 @abstract		Integer that indicates the row count for the insert (array?).
	 @discussion	Contains the data from DXF entity group code 71. This is an integer value indicating the
	 number of rows in the insert (array?).
	 
	 Group code 71:  row count, integer, default = 1
	 */
	int rowCount;	

	/*!
	 @abstract		Double value column spacing.
	 @discussion	Contains the data from DXF entity group code 44. This is a double value indicating the column spacing.
	 
	 Group code 44:  column spacing, double, default = 0
	 */
	double columnSpacing;
	
	/*!
	 @abstract		Double value row spacing.
	 @discussion	Contains the data from DXF entity group code 45. This is a double value indicating the row spacing.
	 
	 Group code 45:  column spacing, double, default = 0
	 */
	double rowSpacing;
	
}

/*!
 @abstract	<i>name</i> instance variable accessible through standard methods.
 */
@property (copy) NSString * name;

/*!
 @abstract	<i>insertionPoint</i> instance variable accessible through standard methods.
 */
@property (retain) BB3DPoint * insertionPoint;

/*!
 @abstract	<i>xScaleFactor</i> instance variable accessible through standard methods.
 */
@property			double		xScaleFactor;

/*!
 @abstract	<i>yScaleFactor</i> instance variable accessible through standard methods.
 */
@property			double		yScaleFactor;

/*!
 @abstract	<i>zScaleFactor</i> instance variable accessible through standard methods.
 */
@property			double		zScaleFactor;

/*!
 @abstract	<i>rotation</i> instance variable accessible through standard methods.
 */
@property			double		rotation;

/*!
 @abstract	<i>columnCount</i> instance variable accessible through standard methods.
 */
@property			int			columnCount;

/*!
 @abstract	<i>rowCount</i> instance variable accessible through standard methods.
 */
@property			int			rowCount;

/*!
 @abstract	<i>columnSpacing</i> instance variable accessible through standard methods.
 */
@property			double		columnSpacing;

/*!
 @abstract	<i>rowSpacing</i> instance variable accessible through standard methods.
 */
@property			double		rowSpacing;

@end
