//
//  BBDXFEntityLine.h
//  DXFReader
//
//  Created by Allan Daly (allan@brown-bird.net) on 1/31/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFEntityLine.h
 
 @abstract		Subclass of BBDXFEntity. Defines class BBDXFEntityLine, which holds the LINE entity data elements.
 
 @discussion	BBDXFEntityLine is an subclass of BBDXFEntity that provides a structure for the data elements
				defined in LINE AutoCAD DXF entities. The data elements in this class plus those in the parent
				class make up all of the data found in the LINE entity definition. (At this time, this implementation
				is not complete. See general discussion for more details.)
 */
#import <Cocoa/Cocoa.h>
#import "BBDXFEntity.h"

@class BB3DPoint;

/*!
 @abstract		Defines abstract class BBDXFEntityLine to hold the LINE DXF entity data.
 
 @discussion	BBDXFEntityLine is a subclass of BBDXFEntity that provides a structure for the data elements
				in LINE AutoCAD DXF entities.
 
				Instance variables defined in BBDXFEntityLine are based on the AutoCAD 2009 DXF Reference
				available at <a href="http://images.autodesk.com/adsk/files/acad_dxf.pdf">http://images.autodesk.com/adsk/files/acad_dxf.pdf</a>
				as of 1/31/2009. The specific names used here were chosen to be consistent with this reference
				and use the AutoCAD terminology wherever possible.
 
				This implementation is not a complete set of the common DXF entity data items.
				See the AutoCAD 2009 DXF Reference listed above for the complete set.
 */
@interface BBDXFEntityLine : BBDXFEntity {

	/*!
	 @abstract		Double value indicating line thickness.
	 @discussion	Contains the data from DXF entity group code 39. This is a double value indicating the line thickness.
	 
	 Group code 39:  thickness, double, default = 0
	 */
	double thickness;			// group code 39: double precision value
								//				  representing entity's thickness if non-zero
								//				  default = 0
	
	/*!
	 @abstract		3D point indicating the line starting point.
	 @discussion	Contains the data from DXF entity group codes 10, 20, 30. This is a 3D point indicating the line starting point. 
	 
	 Group codes 10, 20, 30:  starting point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * startPoint;		// group codes 10, 20, 30 indicating x, y, z values for start point

	/*!
	 @abstract		3D point indicating the line ending point.
	 @discussion	Contains the data from DXF entity group codes 11, 21, 31. This is a 3D point indicating the line ending point. 
	 
	 Group codes 11, 21, 31:  ending point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * endPoint;		// group codes 11, 21, 31 indicating x, y, z values for end point
}

/*!
 @abstract	<i>thickness</i> instance variable accessible through standard methods.
 */
@property double thickness;

/*!
 @abstract	<i>startPoint</i> instance variable accessible through standard methods.
 */
@property (retain) BB3DPoint * startPoint;

/*!
 @abstract	<i>endPoint</i> instance variable accessible through standard methods.
 */
@property (retain) BB3DPoint * endPoint;

/*!
    @method     initWithStartPoint:andEndPoint:
    @abstract   ￼Convenience factory method for creating a line with a start and end point.
    @param      startPt ￼BB3DPoint indicating the line starting point.
    @param      endPt ￼BB3DPoint indicating the line ending point.
    @result     ￼BBDXFEntityLine reference
*/
+ (BBDXFEntityLine *) initWithStartPoint: (BB3DPoint *) startPt andEndPoint: (BB3DPoint *) endPt;

@end
