//
//  BBDXFEntityPolyline.h
//  DXFReader
//
//  Created by Allan Daly (allan@brown-bird.net) on 1/31/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFEntityPolyline.h
 
 @abstract		Subclass of BBDXFEntity. Defines class BBDXFEntityPolyline, which holds the LWPOLYLINE entity data elements.
 
 @discussion	BBDXFEntityPolyline is an subclass of BBDXFEntity that provides a structure for the data elements
				defined in LWPOLYLINE AutoCAD DXF entities. The data elements in this class plus those in the parent
				class make up all of the data found in the LWPOLYLINE entity definition. (At this time, this implementation
				is not complete. See general discussion for more details.)
 */
#import <Cocoa/Cocoa.h>
#import "BBDXFEntity.h"

@class BB3DPoint;

/*!
 @abstract		Defines abstract class BBDXFEntityPolyline to hold the LWPOLYLINE DXF entity data.
 
 @discussion	BBDXFEntityPolyline is a subclass of BBDXFEntity that provides a structure for the data elements
				in LWPOLYLINE AutoCAD DXF entities.
 
				Instance variables defined in BBDXFEntityPolyline are based on the AutoCAD 2009 DXF Reference
				available at <a href="http://images.autodesk.com/adsk/files/acad_dxf.pdf">http://images.autodesk.com/adsk/files/acad_dxf.pdf</a>
				as of 1/31/2009. The specific names used here were chosen to be consistent with this reference
				and use the AutoCAD terminology wherever possible.
 
				This implementation is not a complete set of the common DXF entity data items.
				See the AutoCAD 2009 DXF Reference listed above for the complete set.

				This class implements the LWPOLYLINE object. The LW refers to "light weight" meaning that it is stored more efficiently than
				the "old" or "regular" POLYLINE objects. LWPOLYLINE has been the standard in AutoCAD products since R14 (over 10 years now).
				POLYLINE objects are no longer used regularly and are not supported here.
 */
@interface BBDXFEntityPolyline : BBDXFEntity {

	/*!
	 @abstract		Long integer that indicates the number of vertices in the polyline.
	 @discussion	Contains the data from DXF entity group code 90. This is a long integer value indicating the
					number of vertices in the polyline.
	 
					Group code 90:  number of vertices, long integer, default = 0
	 */
	long int numberOfVertices;	// group code 90: a 32-bit (long) integer value
								//				  indicating the number of vertices in the polyline
	
	/*!
	 @abstract		Integer that provides information about the polyline.
	 @discussion	Contains the data from DXF entity group code 70. This is an integer value indicating the line properties.
	 
	 Group code 70:  thickness, double, default = 0, called the "flag" value by AutoDesk
					<li>a 16-bit integer value</li>
					<li>indicating the polyline "flag" value</li>
					<li>0 = default</li>
					<li>1 = closed</li>
					<li>128 = Plinegen (?? what does this mean ??)</li>
	 */
	int polylineFlag;			// group code 70: a 16-bit integer value
								//				  indicating the polyline "flag" value
								//				  0 = default
								//				  1 = closed
								//				  128 = Plinegen (?? what does this mean ??)
	
	/*!
	 @abstract		Long integer that indicates the constant width of the polyline.
	 @discussion	Contains the data from DXF entity group code 43. This is a long integer value indicating the
	 number of vertices in the polyline.
	 
	 Group code 43:  number of line constand width, long integer, default = 0
	 
	 Note: variable width polylines (group codes 40 and/or 41) not supported at this time.
	 */
	long int constantWidth;		// group code 43: a 32-bit (long) integer value
								//				  indicating the width of the polyline
								//				  note: variable width polyines not supported (codes 40 and/or 41)
	
	/*!
	 @abstract		An array that holds the vertices of the polyline.
	 @discussion	A polyline is composed of straight lines between vertices. This array holds the vertices
					of the polyline. This is a mutable array because as the DXF file is read the vertices are
					added to this array. The array contains BB3DPoint objects.
	 */
	NSMutableArray * vertices;	// contains an array of BB3DPoint objects indicating the polyline vertices
}

/*!
 @abstract	<i>numberOfVertices</i> instance variable accessible through standard methods.
 */
@property long int numberOfVertices;

/*!
 @abstract	<i>polylineFlag</i> instance variable accessible through standard methods.
 */
@property int polylineFlag;

/*!
 @abstract	<i>constantWidth</i> instance variable accessible through standard methods.
 */
@property long constantWidth;

/*!
    @method     addVertex:
    @abstract   ￼Method for adding a vertex to the vertices array.
    @param      newVertex 3DBBPoint object to be added to the vertices array.￼
*/
- (void) addVertex: (BB3DPoint *) newVertex;

/*!
    @method     vertexAtIndex:
    @abstract   ￼Method for accessing a vertex from the vertices array.
    @param      index ￼Integer value indicating which vertex to return.
    @result     ￼BB3DPoint indicating the vertex coordinates.
*/
- (BB3DPoint *) vertexAtIndex: (int) index;

/*!
    @method     vertexArray
    @abstract   ￼Method for accessing the entire vertex array.
    @result     ￼NSArray reference to the vertices array owned by this object.
*/
- (NSArray *) vertexArray;

@end
