//
//  BBDXFEntityText.h
//  DXFReader
//
//  Created by Allan Daly (allan@brown-bird.net) on 2/2/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//

// header file HeaderDoc info
/*!
 @header		BBDXFEntityText.h
 
 @abstract		Subclass of BBDXFEntity. Defines class BBDXFEntityText, which holds the TEXT entity data elements.
 
 @discussion	BBDXFEntityText is an subclass of BBDXFEntity that provides a structure for the data elements
				defined in TEXT AutoCAD DXF entities. The data elements in this class plus those in the parent
				class make up all of the data found in the TEXT entity definition. (At this time, this implementation
				is not complete. See general discussion for more details.)
 */
#import <Cocoa/Cocoa.h>
#import "BBDXFEntity.h"

@class BB3DPoint;

/*!
 @abstract		Defines abstract class BBDXFEntityText to hold the TEXT DXF entity data.
 
 @discussion	BBDXFEntityText is a subclass of BBDXFEntity that provides a structure for the data elements
				in TEXT AutoCAD DXF entities.
 
				Instance variables defined in BBDXFEntityText are based on the AutoCAD 2009 DXF Reference
				available at <a href="http://images.autodesk.com/adsk/files/acad_dxf.pdf">http://images.autodesk.com/adsk/files/acad_dxf.pdf</a>
				as of 1/31/2009. The specific names used here were chosen to be consistent with this reference
				and use the AutoCAD terminology wherever possible.
 
				This implementation is not a complete set of the common DXF entity data items.
				See the AutoCAD 2009 DXF Reference listed above for the complete set.
 */
@interface BBDXFEntityText : BBDXFEntity {
	
	/*!
	 @abstract		Double value indicating line thickness.
	 @discussion	Contains the data from DXF entity group code 39. This is a double value indicating the line thickness.
	 
	 Group code 39:  thickness, double, default = 0
	 */
	double thickness;			// group code 39: double precision value
								//				  representing entity's thickness if non-zero
								//				  default = 0
	
	/*!
	 @abstract		3D point indicating the first alignment point for the text.
	 @discussion	Contains the data from DXF entity group codes 10, 20, 30. This is a 3D point indicating the first alignment point.
	 
	 The first and second alignment points are used in different ways depending on the horizontal and vertical alignment settings. 
	 xactly how they are used is relatively complicated. See the AutoCAD DXF specification for more information.
	 
	 Group codes 10, 20, 30:  first alignment point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * firstAlignmentPoint;	// group codes 10, 20, 30 indicating x, y, z values for first alignment point
	
	/*!
	 @abstract		3D point indicating the second alignment point for the text.
	 @discussion	Contains the data from DXF entity group codes 10, 20, 30. This is a 3D point indicating the second alignment point.
	 
	 The first and second alignment points are used in different ways depending on the horizontal and vertical alignment settings. 
	 xactly how they are used is relatively complicated. See the AutoCAD DXF specification for more information.
	 
	 Group codes 11, 21, 31:  second alignment point, doubles, no default, but BB3DPoint objects are iniatilized to 0, 0, 0
	 */
	BB3DPoint * secondAlignmentPoint;	// group codes 11, 21, 31 indicating x, y, z values for second alignment point
	
	/*!
	 @abstract		Double value indicating text height [units?].
	 @discussion	Contains the data from DXF entity group code 40. This is a double value indicating the text height [what units?].
	 
	 Group code 40:  text height, double, default = 0
	 */
	double textHeight;			// group code 40: double precision value
								//				  representing the text height (what units?)
								//				  [ASSUME THIS IS THE FONT SIZE?]

	/*!
	 @abstract		NSString containing the text value itself.
	 @discussion	Contains the data from DXF entity group code 1. This is a string value indicating the text value itself.
	 
	 Group code 1:  text string, no default
	 */
	NSString * textString;		// group code 1:  the text value itself
	
	/*!
	 @abstract		NSString containing the text style name.
	 @discussion	Contains the data from DXF entity group code 7. This is a string value indicating the text style name.
	 
	 Group code 7:  text string, no default
	 */
	NSString * textStyleName;	// group code 7:  the name of the text style
								//				  default = "STANDARD"
	
	/*!
	 @abstract		Double value indicating text rotation (in degrees).
	 @discussion	Contains the data from DXF entity group code 50. This is a double value indicating the text rotation (in degrees).
	 
	 Group code 50:  text rotation (in degrees), double, default = 0
	 */
	double textRotation;		// group code 50: double precision value
								//				  rotation in degrees
								//				  default = 0

	/*!
	 @abstract		Double value x scale factor.
	 @discussion	Contains the data from DXF entity group code 41. This is a double value indicating the x scale factor.
	 
	 Group code 41:  x scale factor, double, default = 1
	 */
	double xScaleFactor;		// group code 41: double precision value
								//				  default = 1
	
	/*!
	 @abstract		Integer that provides information about the text properties.
	 @discussion	Contains the data from DXF entity group code 71. This is an integer value indicating the text properties.
	 
	 Group code 71:  flags, integer, default = 0, called the "flag" value by AutoDesk
	 <li>a 16-bit integer value</li>
	 <li>indicating the text "flag" value</li>
	 <li>0 = default</li>
	 <li>2 = text is backwards (mirrored in X)</li>
	 <li>4 = text is upside-down (mirrored in Y)</li>
	 */
	int textGenerationFlags;	// group code 71: 0 = default
								//				  2 = text is backwards (mirrored in X)
								//				  4 = text is upside-down (mirrored in Y)
	
	/*!
	 @abstract		Integer that provides information about the text horizontal alignment.
	 @discussion	Contains the data from DXF entity group code 72. This is an integer value indicating the text horizontal alignment.
	 
	 Group code 72:  horizontal alignment, integer, default = 0
	 <li>a 16-bit integer value</li>
	 <li>indicating the text horizontal alignment value</li>
	 <li>0 = left (default)</li>
	 <li>1 = center</li>
	 <li>2 = right</li>
	 <li>3 = aligned (if vetical alignment = 0)</li>
	 <li>4 = middle (if vertical alignment = 0)</li>
	 <li>5 = fit (if vertical alignment = 0)</li>
	 */
	int horizontalJustification;	// group code 72: 0 = left, 1 = center, 2 = right
									//				  3 = aligned (if vetical alignment = 0)
									//				  4 = middle (if vertical alignment = 0)
									//				  5 = fit (if vertical alignment = 0)
	
	/*!
	 @abstract		Integer that provides information about the text vertical alignment.
	 @discussion	Contains the data from DXF entity group code 73. This is an integer value indicating the text vertical alignment.
	 
	 Group code 73:  vertical alignment, integer, default = 0
	 <li>a 16-bit integer value</li>
	 <li>indicating the text horizontal alignment value</li>
	 <li>0 = baseline (default)</li>
	 <li>1 = bottom</li>
	 <li>2 = middle</li>
	 <li>3 = top</li>
	 */
	int verticalJustification;		// group code 73: 0 = baseline, 1 = bottom, 2 = middle, 3 = top
									//				  default = 0
}


/*!
 @abstract	<i>thickness</i> instance variable accessible through standard methods.
 */
@property			double		thickness;

/*!
 @abstract	<i>firstAlignmentPoint</i> instance variable accessible through standard methods.
 */
@property (retain)	BB3DPoint *	firstAlignmentPoint;

/*!
 @abstract	<i>secondAlignmentPoint</i> instance variable accessible through standard methods.
 */
@property (retain)	BB3DPoint *	secondAlignmentPoint;

/*!
 @abstract	<i>textHeight</i> instance variable accessible through standard methods.
 */
@property			double		textHeight;

/*!
 @abstract	<i>textString</i> instance variable accessible through standard methods.
 */
@property (retain)	NSString *	textString;

/*!
 @abstract	<i>textStyleName</i> instance variable accessible through standard methods.
 */
@property (retain)	NSString *	textStyleName;

/*!
 @abstract	<i>textRotation</i> instance variable accessible through standard methods.
 */
@property			double		textRotation;

/*!
 @abstract	<i>xScaleFactor</i> instance variable accessible through standard methods.
 */
@property			double		xScaleFactor;

/*!
 @abstract	<i>textGenerationFlags</i> instance variable accessible through standard methods.
 */
@property			int			textGenerationFlags;

/*!
 @abstract	<i>horizontalJustification</i> instance variable accessible through standard methods.
 */
@property			int			horizontalJustification;

/*!
 @abstract	<i>verticalJustification</i> instance variable accessible through standard methods.
 */
@property			int			verticalJustification;

@end
