//
//  BBDXFParser.h
//  DXFReader
//
//  Created by Michael Caron (michael.r.caron@gmail.com) on 3/14/09.
//    Based on previous implementation by Allan Daly (allan@brow-bird.net) on 1/30/09.
//	For information related to this framework see brown-bird.net.
//
//	********************************************************************************
//	Copyright (c) 2009 Allan Daly, Michael Caron
//	All rights reserved.
//	
//	Redistribution and use in source and binary forms, with or without
//	modification, are permitted provided that the following conditions are met:
//	    * Redistributions of source code must retain the above copyright
//	      notice, this list of conditions and the following disclaimer.
//	    * Redistributions in binary form must reproduce the above copyright
//	      notice, this list of conditions and the following disclaimer in the
//	      documentation and/or other materials provided with the distribution.
//	    * Neither the name of Brown Bird nor the
//	      names of its contributors may be used to endorse or promote products
//	      derived from this software without specific prior written permission.
//	
//	THIS SOFTWARE IS PROVIDED BY ALLAN DALY, MICHAEL CARON ''AS IS'' AND ANY
//	EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL ALLAN DALY, MICHAEL CARON BE LIABLE FOR ANY
//	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
//	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//	********************************************************************************
//


// header file HeaderDoc info
/*!
 @header		BBDXFParser.h
 @abstract		Defines the BBDXFParser to represent the data in an ASCII DXF file.
 @discussion	This header defines class BBDXFParser.
 
 BBDXFParser encapsulates the DXF File parsing logic used to populate a BBDXFModel. 
 It receives its data from a text file or string formatted based on the AutoCAD DXF
 spec.  It generates custom objects for each element in the DXF File and adds them
 to the model using the Model's public interface.
 */
#import <Cocoa/Cocoa.h>

// DXFDataPair object HeaderDoc info
/*!
 @abstract		Defines a data pair for use in describing a code and value in a
				DXF file
 @discussion	DXF files are based on a basic format of two lines, a DXF Group Code
				followed by a Value for that code. This class represents the
				line pair as an object for which code and value can be retrieved.
 */
@interface DXFDataPair : NSObject
{
	int			dxfGroupCode;
	NSString*	dxfStringValue;
}

#pragma mark Properties
@property				int				dxfGroupCode;
@property(assign)		NSString*		dxfStringValue;

@end

// BBDXFParser object HeaderDoc info
/*!
 @abstract		A BBDXFParser object defines how a DXF File is parsed into logical pieces
					that are added to a BBDXFModel
 @discussion	The BBDXFParser works by parsing DXFDataPairs (pairs of lines) from a DXF file.
					For each pair, it tries to parse a DXF element, that is an object or an
					entity. It works with the Autodesk 2009 DXF format and currently processes
					the HEADER, BLOCKS and ENTITIES sections of a DXF file.
 */
@interface BBDXFParser : NSObject {
	NSArray*		dxfFileContents;
	int				currentIndex;
	DXFDataPair*	currentDXFDataPair; 
	BBDXFModel*		model;
	NSString*		currentSection;
}

#pragma mark Properties
@property(readonly)			NSArray*		dxfFileContents;
@property(readonly)			int				currentIndex;

/*!
 @abstract						Read-Only interface to the DXFDataPair's string value
 @result						The string value associated with the group code
 */
-(NSString*)	currentStringValue;  

/*!
 @abstract						Read-Only interface to the DXFDataPair's group code
 @result						The group code as an integer
 */
-(int)			currentGroupCode;    

#pragma mark Initialization Methods
/*!
 @abstract						Initializes a parser with a model to receive elements
 @discussion					The model passed into this method becomes the receiver of parse events. For example, when
									a parser parses a LINE entity, it will create the LINE entity and tell the model to
									add a new line with that object.
 @result						a new parser object
 @updated 2009-03-14
 */
- (id) initWithModel: (BBDXFModel*) m;

#pragma mark DXF File Interaction Methods
/*!
 @abstract   ￼					Loads the DXF file into memory.
 @param			filename		An NSString containing the full path including filename to the DXF file to load.
 @discussion ￼					This method is called to load the DXF file into memory so it can be processed by the 
									other DXF parser methods. It may be that as the DXFReader framework is developed further
									that it does not make sense to keep this as a stand-alone method. It might make more
									sense instead to have this code be part of a more complete DXF processing procedure.
 @result						￼BOOL: YES means the load succeeded, NO means it failed
 @updated	2009-02-14
 */
- (BOOL) loadFile: (NSString *) filename;	// brings the DXF file into memory

/*!
 @abstract						Loads a DXF file represented by an NSData entity into memory. 
 @param			fileContents	An NSData object containing the file contents of a DXF file.
 @discussion ￼					This method is called to load the DXF contents into memory so it can be processed by the 
									other DXF parser methods. It may be that as the DXFReader framework is developed further
									that it does not make sense to keep this as a stand-alone method. It might make more
									sense instead to have this code be part of a more complete DXF processing procedure.
 @result					￼	BOOL: YES means the load succeeded, NO means it failed
 @updated	2009-02-25
 */
- (BOOL) loadString: (NSString *) fileContents; // brings the DXF file data into memory

#pragma mark Parser Control Methods
/*!
 @abstract						Parses the entire file contents
 @discussion					After a parse operation, your model will be filled with the appropriate information, header,
									entities, etc. and can be accessed with those model methods.
 @result						void
 @updated	2009-03-14
 */
- (void) parse;

/*!
 @abstract						Sets the parser at the beginning of the file contents
 @discussion					Next time forward is called, it will be the first element pair in the contents
 @result						void
 @updated	2009-03-14
 */
- (void) reset;

/*!
 @abstract						Moves the parser to the next Group/Value pair
 @result						void, the result is the next time getDXFDataPair is called, it will return the next gid/val pair
 */
- (void) next;

/*!
 @abstract						Moves the parser to the previous Group/Value pair
 @discussion					The result is the next time getDXFDataPair is called, it will return the previous gid/val pair
								and the result of the following will be the present DXFDataPair:
									DXFDXFDataPair t1 = [p getDXFDataPair];
									[p previous];
									[p next];
									DXFDXFDataPair t2 = [p getDXFDataPair];
									// t1 and t2 are the same
 @result						void
 */
- (void) previous;

/*!
 @abstract						Tells whether you're at the end of the file or not
 @result						True if at end of file
 */
- (BOOL) isAtEnd;

@end

